/*-----------------------------------------------------------------------------
 * Copyright (c) 2015-2021 Arm Limited (or its affiliates). All
 * rights reserved.
 *
 * SPDX-License-Identifier: BSD-3-Clause
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *   1.Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *   2.Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *   3.Neither the name of Arm nor the names of its contributors may be used
 *     to endorse or promote products derived from this software without
 *     specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL COPYRIGHT HOLDERS AND CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *-----------------------------------------------------------------------------
 * Name:    Buttons_NUCLEO_F746ZG.c
 * Purpose: Buttons interface for NUCLEO-F746ZG board
 * Rev.:    1.0.0
 *----------------------------------------------------------------------------*/

#include "stm32f7xx_hal.h"
#include "Board_Buttons.h"

/* GPIO Pin configuration structure */
typedef struct {
  GPIO_TypeDef *port;
  uint16_t      pin;
  GPIO_PinState inactive_state;
  GPIO_PinState active_state;
} GPIO_PIN;

/* Buttons GPIO Pins Clock Enable */
#define LED_CLOCK_ENABLE()              __HAL_RCC_GPIOC_CLK_ENABLE()

/* Buttons GPIO Pins */
static const GPIO_PIN BUTTON_PIN[] = {
  { GPIOC, GPIO_PIN_13, GPIO_PIN_RESET, GPIO_PIN_SET }
};

#define BUTTONS_COUNT (sizeof(BUTTON_PIN)/sizeof(GPIO_PIN))


/**
  \fn          int32_t Buttons_Initialize (void)
  \brief       Initialize buttons
  \returns
   - \b  0: function succeeded
   - \b -1: function failed
*/
int32_t Buttons_Initialize (void) {
  GPIO_InitTypeDef GPIO_InitStruct;
  uint8_t          num;
  
  /* Enable Buttons GPIO Pins Clock */
  LED_CLOCK_ENABLE();

  /* Configure GPIO LED pins */
  GPIO_InitStruct.Mode  = GPIO_MODE_INPUT;
  GPIO_InitStruct.Pull  = GPIO_NOPULL;
  for (num = 0U; num < BUTTONS_COUNT; num++) {
    GPIO_InitStruct.Pin = BUTTON_PIN[num].pin;
    HAL_GPIO_Init(BUTTON_PIN[num].port, &GPIO_InitStruct);
  }

  return 0;
}

/**
  \fn          int32_t Buttons_Uninitialize (void)
  \brief       De-initialize buttons
  \returns
   - \b  0: function succeeded
   - \b -1: function failed
*/
int32_t Buttons_Uninitialize (void) {
  uint8_t num;

  for (num = 0U; num < BUTTONS_COUNT; num++) {
    HAL_GPIO_DeInit(BUTTON_PIN[num].port, BUTTON_PIN[num].pin);
  }

  return 0;
}

/**
  \fn          uint32_t Buttons_GetState (void)
  \brief       Get buttons state
  \returns     Buttons state
*/
uint32_t Buttons_GetState (void) {
  uint32_t val;
  uint8_t  num;

  val = 0U;

  for (num = 0U; num < BUTTONS_COUNT; num++) {
    if (HAL_GPIO_ReadPin(BUTTON_PIN[num].port, BUTTON_PIN[num].pin) == BUTTON_PIN[num].active_state) {
      val |= (1U << num);
    }
  }

  return val;
}

/**
  \fn          uint32_t Buttons_GetCount (void)
  \brief       Get number of available buttons
  \return      Number of available buttons
*/
uint32_t Buttons_GetCount (void) {
  return BUTTONS_COUNT;
}
